/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.reporting;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.admin.setting.SettingRepository;

class ReportingAggregationManagerSettingDtoServiceTest {

	@Spy
	@InjectMocks
	private ReportingAggregationManagerSettingDtoService service;

	@Mock
	private AggregationMappingMapper mapper;
	@Mock
	private SettingRepository repository;

	@Nested
	class TestMapToDto {

		@BeforeEach
		void setupMapperMock() {
			when(mapper.toMapping(any())).thenReturn(AggregationMappingDtoTestFactory.create());
		}

		@Test
		void shouldCallMapper() {
			var mapping = AggregationMappingTestFactory.create();
			service.mapToDto(Stream.of(mapping));

			verify(mapper).toMapping(mapping);
		}

		@Test
		void shouldBuildDto() {
			var dto = service.mapToDto(Stream.of(AggregationMappingTestFactory.create()));

			assertThat(dto).usingRecursiveComparison().isEqualTo(ReportingAggregationMAnagerSettingDtoTestFactory.create());
		}
	}

	@Nested
	class TestGetDataTransferObject {

		@Captor
		private ArgumentCaptor<Stream<AggregationMapping>> streamCaptor;

		@BeforeEach
		void disableMapToDtoFunction() {
			doReturn(ReportingAggregationMAnagerSettingDtoTestFactory.create()).when(service).mapToDto(any());
		}

		@Test
		void shouldCallRepository() {
			service.getDataTransferObject();

			verify(repository).findByType(AggregationMapping.TYPE_ALIAS, AggregationMapping.class);
		}

		@Test
		void shouldCallMapToDto() {
			AggregationMapping mapping = AggregationMappingTestFactory.create();
			when(repository.findByType(any(), any())).thenReturn(Stream.of(mapping));

			service.getDataTransferObject();

			verify(service).mapToDto(streamCaptor.capture());
			assertThat(streamCaptor.getValue()).contains(mapping);
		}
	}

}
